<?php

namespace NoccyLabs\DevMode;

class Application
{

    protected $plugins = [];

    public function __construct()
    {
        $paths = [
            __DIR__."/../plugins"
        ];
        foreach ($paths as $path) {
            if (is_dir($path)) {
                $this->loadPlugins($path);
            }
        }
    }

    public function run()
    {
        global $argv;
        list($opts,$args) = \Console_Getopt::getopt($argv, "hvV", [ "help", "verbose", "version" ]);

        if (count($args) == 0) {
            $this->showUsage();
            return EXIT_BAD_COMMAND;
        }
        foreach ($opts as list($opt,$optval)) {
            switch ($opt) {
                case 'h':
                case 'help':
                    $this->showUsage();
                    return EXIT_OK;
                case 'v':
                case 'verbose':
                    $verbosity = min($verbosity+1,3);
                    break;
                case 'V':
                case 'version':
                    $this->showVersion();
                    return EXIT_OK;
            }
        }

        $manifest = new Manifest();

        $action = array_shift($args);
        switch (strtolower($action)) {
            case 'up':
            case 'down':
            case 'status':
                $this->action($manifest, strtolower($action));
                return EXIT_OK;
            default:
                throw new \RuntimeException("Bad action: {$action}");
        }
    }

    private function showUsage()
    {
        printf("devmode [options] action [args...]\n");

        $options = [
            "-h,--help" => "Show this help",
            "-v[vv],--verbose" => "Verbose(r) output",
            "-V,--version" => "Print version and exit",
        ];
        printf("\nOptions:\n");
        foreach ($options as $option=>$info) {
            printf("  %-20s %s\n", $option, $info);
        }

        $actions = [
            "up" => "Start all defined services",
            "down" => "Stop all defined services",
            "status" => "Show service status",
        ];
        printf("\nActions:\n");
        foreach ($actions as $action=>$info) {
            printf("  %-15s %s\n", $action, $info);
        }

    }
    
    private function showVersion()
    {
        printf("devmode v0.1-alpha\n");
    }
    
    private function action(Manifest $manifest, $action)
    {
        $deps = $manifest->getDependencies();
        foreach ($deps as $dependency) {
            $dm = $this->getDependencyManager($dependency);
            switch ($action) {
                case 'up':
                    $dm->setDependencyState($dependency, true);
                    break;
                case 'down':
                    $dm->setDependencyState($dependency, false);
                    break;
                case 'status':
                    $dm->showDependencyStatus($dependency);
                    break;
            }
        }
    }

    public function getPluginsByInterface($interface)
    {
        $plugins = [];
        foreach ($this->plugins as $plugin) {
            if ($plugin instanceof $interface) {
                $plugins[] = $plugin;
            }
        }
        return $plugins;
    }
    
    private function getDependencyManager(Dependency $dependency)
    {
        $plugins = $this->getPluginsByInterface(Plugin\IDependencyManager::class);
        $manager = null; $managerWeight = null;
        foreach ($this->plugins as $plugin) {
            $w = $plugin->isManagedDependency($dependency);
            if ($w == false) {
                continue;
            }
            if ((float)$w > $managerWeight) {
                $manager = $plugin;
                $managerWeight = (float)$w;
            }
        }
        if (!$manager) {
            throw new \RuntimeException("Could not find a manager for {$dependency->getId()}");
        }
        return $manager;
    }

    private function loadPlugins($path)
    {
        $plugins = glob($path."/*");
        foreach ($plugins as $dir) {
            if (!file_exists($dir."/plugin.json")) {
                continue;
            }
            $pluginManifest = json_decode(file_get_contents($dir."/plugin.json"));
            $pluginClass = require_once $dir."/plugin.php";
            $pluginClass->setManifest($pluginManifest);
            if ($pluginClass->onLoad() === false) {
                continue;
            }
            $this->plugins[$pluginManifest->plugin] = $pluginClass;
        }
    }

}
